#! /bin/bash
# socat server

NAME="socat_server"
PORT=$1

. /etc/default/socat

KEY="${PORT}_MODE"; MODE="${!KEY^^}"

KEY="${PORT}_PROTO"; PROTO="${!KEY^^}"
KEY="${PORT}_LCLPORT"; LCLPORT="${!KEY}"
KEY="${PORT}_DSTPORT"; DSTPORT="${!KEY}"
KEY="${PORT}_DSTADDR"; DSTADDR="${!KEY}"

KEY="${PORT}_WAIT"; WAIT="${!KEY}"

# TCP options
KEY="${PORT}_KEEPCNT"; KEEPCNT="${!KEY}"
KEY="${PORT}_KEEPIDLE"; KEEPIDLE="${!KEY}"
KEY="${PORT}_KEEPINTVL"; KEEPINTVL="${!KEY}"
KEY="${PORT}_SNDTIMEO"; SNDTIMEO="${!KEY}"
KEY="${PORT}_TCPCORK"; TCPCORK="${!KEY}"

KEY="${PORT}"; UART="${!KEY}"
KEY="${PORT}_BAUD"; BAUD="${!KEY}"
KEY="${PORT}_CHAR"; CHAR="${!KEY}"
KEY="${PORT}_STOP"; STOP="${!KEY}"
KEY="${PORT}_PARITY"; PARITY="${!KEY}"
KEY="${PORT}_RTSCTS"; RTSCTS="${!KEY^^}"
KEY="${PORT}_TIME"; TIME="${!KEY}"
KEY="${PORT}_MIN"; MIN="${!KEY}"

if [ "$((WAIT+0))" = "0" ]; then
  WAIT=60
fi

if [ "$((KEEPCNT+0))" = "0" ]; then
  KEEPCNT=3
fi

if [ "$((KEEPIDLE+0))" = "0" ]; then
  KEEPIDLE=120
fi

if [ "$((KEEPINTVL+0))" = "0" ]; then
  KEEPINTVL=120
fi

if [ "$((SNDTIMEO+0))" = "0" ]; then
  SNDTIMEO=10
fi

# TCP_USER_TIMEOUT [ms]
USER_TIMEOUT=$((SNDTIMEO*1000))

case $CHAR in
8|7)
  CS="cs${CHAR}"
  ;;
*)
  CS="cs8"
  ;;
esac

if [ "$PARITY" = "NONE" ]; then
  PARITYOPTS="parenb=0"
elif [ "$PARITY" = "EVEN" ]; then
  PARITYOPTS="parenb=1,parodd=0"
elif [ "$PARITY" = "ODD" ]; then
  PARITYOPTS="parenb=1,parodd=1"
fi

CSTOPB=$((STOP-1))

if [ "$RTSCTS" = "YES" ]; then
  CRTSCTS=1
else
  CRTSCTS=0
fi

if [ "$TIME" = "" ]; then
  TIME=1
fi

if [ "$MIN" = "" ]; then
  MIN=1
fi

cmdline=""
scan_before=0

UARTOPTS="b${BAUD},${CS},cstopb=${CSTOPB},crtscts=${CRTSCTS},raw,echo=0,${PARITYOPTS},time=${TIME},min=${MIN}"

if [ "$MODE" = "SERVER" ]; then
  SOCKOPTS="reuseaddr,fork"
  if [ "$PROTO" = "TCP" ]; then
    SOCKOPTS="${SOCKOPTS},keepalive,keepcnt=${KEEPCNT},keepidle=${KEEPIDLE},keepintvl=${KEEPINTVL},setsockopt-int=6:18:${USER_TIMEOUT}"
    if [ "$TCPCORK" != "0" ]; then
      SOCKOPTS="${SOCKOPTS},cork"
    fi
  fi
  TTYOPTS="raw,echo=0"
  cmdline="socat -d ${PROTO}-LISTEN:${LCLPORT},${SOCKOPTS} EXEC:'\"/usr/local/sbin/socat_child ${PORT} ${UARTOPTS} ${TTYOPTS}\"',pty,raw,echo=0"
elif [ "$MODE" = "CLIENT" ]; then
  if [ "$PROTO" = "TCP" ]; then
    SOCKOPTS="keepalive,keepcnt=${KEEPCNT},keepidle=${KEEPIDLE},keepintvl=${KEEPINTVL},setsockopt-int=6:18:${USER_TIMEOUT}"
    if [ "$TCPCORK" != "0" ]; then
      SOCKOPTS="${SOCKOPTS},cork"
    fi
    cmdline="socat -d /dev/${PORT},${UARTOPTS} tcp:${DSTADDR}:${DSTPORT},${SOCKOPTS}"
    scan_before=1
  elif [ "$PROTO" = "UDP" ]; then
    cmdline="socat -d /dev/${PORT},${UARTOPTS} udp:${DSTADDR}:${DSTPORT}"
    if [ "$LCLPORT" != "" ]; then
      cmdline="${cmdline},sourceport=${LCLPORT}"
    fi
  fi
fi

if [ "$scan_before" = "1" ]; then
  # TCP Client mode only
  while true; do
    nc -z -w5 ${DSTADDR} ${DSTPORT}
    if [ "$?" = "0" ]; then
      logger -t $NAME "remote (${DSTADDR}:${DSTPORT}) is OK."
      sleep 2
      break
    else
      logger -t $NAME "remote (${DSTADDR}:${DSTPORT}) is closed."
      sleep $WAIT
    fi
  done
fi

if [ "$cmdline" != "" ]; then
  logger -t $NAME "cmdline: ${cmdline}"
  eval $cmdline
fi
